#include "ast.h"
#include <string.h>

// **********************************************************************
// Constructors for all of the subclasses of ASTnode that need one
// **********************************************************************

DeclListNode::DeclListNode(list<DeclNode *> *L): myKids(L)
{}

FormalsListNode::FormalsListNode(list<FormalNode *> *L): myKids(L)
{}

FnBodyNode::FnBodyNode(DeclListNode *declList, StmtListNode *stmtList):
   myDeclList(declList), myStmtList(stmtList)
{}

StmtListNode::StmtListNode(list<StmtNode *> *L): myKids(L)
{}

ExpListNode::ExpListNode(list<ExpNode *> *L): myKids(L)
{}

VarDeclNode::VarDeclNode(TypeNode * type, IdNode *id):
  myType(type), myId(id)
{}

FnDeclNode::FnDeclNode(IdNode *id, FormalsListNode *formalList,
		       FnBodyNode *body):
   myId(id), myFormalsList(formalList), myBody(body)
{}

FormalNode::FormalNode(TypeNode *type, IdNode *id):
   myType(type), myId(id)
{}

IntegerNode::IntegerNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

FloatNode::FloatNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

BooleanNode::BooleanNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

StringNode::StringNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

WriteStmtNode::WriteStmtNode(ExpNode *exp): myExp(exp)
{}

AssignStmtNode::AssignStmtNode(IdNode *id, ExpNode *exp):
   myId(id), myExp(exp)
{}

IfStmtNode::IfStmtNode(ExpNode *exp, StmtListNode *stmtList):
   myExp(exp), myStmtList(stmtList)
{}

IfElseStmtNode::IfElseStmtNode(ExpNode *exp, StmtListNode *thenStmtList,
			       StmtListNode *elseStmtList):
   myExp(exp), myThenStmtList(thenStmtList), myElseStmtList(elseStmtList)
{}

WhileStmtNode::WhileStmtNode(ExpNode *exp, StmtListNode *stmtList):
   myExp(exp), myStmtList(stmtList)
{}

CallStmtNode::CallStmtNode(IdNode *id, ExpListNode *expList):
   myId(id), myExpList(expList)
{}

ReturnStmtNode::ReturnStmtNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

IntLitNode::IntLitNode(int lineNum, int colNum, int intVal):
  myLineNum(lineNum), myColNum(colNum), myIntVal(intVal)
{}

FloatLitNode::FloatLitNode(int lineNum, int colNum, float floatVal):
  myLineNum(lineNum), myColNum(colNum), myFloatVal(floatVal)
{}

StrLitNode::StrLitNode(int lineNum, int colNum, string *strVal):
  myLineNum(lineNum), myColNum(colNum), myStrVal(strVal)
{}

TrueNode::TrueNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

FalseNode::FalseNode(int lineNum, int colNum):
  myLineNum(lineNum), myColNum(colNum)
{}

IdNode::IdNode(int lineNum, int colNum, string *strVal):
  myLineNum(lineNum), myColNum(colNum), myStrVal(strVal)
{}

// Unary operators -----------------------------------
UnaryMinusExpNode::UnaryMinusExpNode(ExpNode *exp):
  myExp(exp)
{}

NotExpNode::NotExpNode(ExpNode *exp):
  myExp(exp)
{}

// Binary operators -----------------------------------
PlusExpNode::PlusExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

MinusExpNode::MinusExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

TimesExpNode::TimesExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

DivideExpNode::DivideExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

AndExpNode::AndExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

OrExpNode::OrExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

EqualsExpNode::EqualsExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

NotEqualsExpNode::NotEqualsExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

LessExpNode::LessExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

GreaterExpNode::GreaterExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

LessEqExpNode::LessEqExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

GreaterEqExpNode::GreaterEqExpNode(ExpNode *exp1, ExpNode *exp2):
  myExp1(exp1), myExp2(exp2)
{}

