#include "ast.h"

// **********************************************************************
// Unparse operations
//
// ***WARNING***
// Versions of the Unparse operation are provided for:
//      DeclListNode, VarDeclNode, IntegerNode, BooleanNode, StringNode,
//      IdNode
// These are not necessarily complete.
//
// Incomplete versions for the other kinds of ASTnode are provided so
// that the code will compile for you; you must replace all the other
// function bodies with the correct code.
//
// Notes:
//  1. Some versions of Unparse do not use their "indentation"
//     parameter; in these cases, have the list of formals include
//     only the type (int), not the name (indent); that way you won't
//     get a warning message from the compiler about an unused parameter.
// **********************************************************************

//ind is a helper function that just prints out spaces...

inline void ind(int indent)
{
	for(int i = 0; i < indent; i++)
		cout << " ";
}

void DeclListNode::Unparse(int indent)
{
  for( list<DeclNode *>::iterator i = myKids->begin(); i != myKids->end(); ++i )
  {
	(*i)->Unparse(indent);
  }
}

void VarDeclNode::Unparse(int indent)
{
  ind(indent);
  myType->Unparse();
  myId->Unparse();
  cout << ";" << endl;
}

void IntegerNode::Unparse(int)
{ cout << "int "; }

void FloatNode::Unparse(int)
{ cout << "float "; }

void BooleanNode::Unparse(int)
{ cout << "bool "; }

void StringNode::Unparse(int)
{ cout << "String "; }

void IdNode::Unparse(int)
{ cout << *myStrVal; }

// EMPTY FUNCTION DEFINITIONS START HERE

void FormalsListNode::Unparse(int indent)
{
	list<FormalNode *>::iterator i;
	for( i = myKids->begin(); i != myKids->end(); )
	{
		(*i)->Unparse(indent);
		if(++i != myKids->end() )
		{
		   cout <<", ";
		}
	}

}

void FnBodyNode::Unparse(int indent)
{
	myDeclList->Unparse(indent+4);
	cout << endl;
	myStmtList->Unparse(indent+4);
}

void StmtListNode::Unparse(int indent)
{
	list<StmtNode *>::iterator i;
	for( i = myKids->begin(); i != myKids->end(); ++i)
	{
		(*i)->Unparse(indent);
	}
}

void ExpListNode::Unparse(int indent)
{
	//ind(indent);
	list<ExpNode *>::iterator i;
	for( i = myKids->begin(); i != myKids->end(); ++i)
	{
		(*i)->Unparse(indent);
	}
	
}

void FnDeclNode::Unparse(int indent)
{
  cout << endl << "void ";
  myId->Unparse();
  cout << "(";
  if(myFormalsList) //don't try and parse it if we ain't got it
  	myFormalsList->Unparse(indent);
  cout << ") {" << endl;
  if(myBody)
  { 
 	 myBody->Unparse(indent+4);
  }	
  cout << "}" << endl << endl;
}

void FormalNode::Unparse(int indent)
{
  myType->Unparse();
  myId->Unparse();
}

void WriteStmtNode::Unparse(int indent)
{
	ind(indent);
	cout << "write(";
	myExp->Unparse();
	cout << ");" << endl;
}

void AssignStmtNode::Unparse(int indent)
{
	ind(indent);
	myId->Unparse();
	cout << " = " ;
	myExp->Unparse();
	cout << ";" << endl;
}

void IfStmtNode::Unparse(int indent)
{
	ind(indent);
	cout << "if(";
	myExp->Unparse();
	cout << ") {" << endl;
	myStmtList->Unparse(indent+4);
	ind(indent);
	cout <<  "}" << endl;
}

void IfElseStmtNode::Unparse(int indent)
{
	ind(indent);
	cout << "if(";
	myExp->Unparse();
	cout << ") {" << endl;
	myThenStmtList->Unparse(indent+4);
	ind(indent);
	cout <<  "}" << endl;
	ind(indent);
	cout << "else {" << endl;
	myElseStmtList->Unparse(indent+4);
	ind(indent);
	cout << "}" << endl;
}

void WhileStmtNode::Unparse(int indent)
{
	ind(indent);
	cout << "while(";
	myExp->Unparse();
	cout << ") {" << endl;
	myStmtList->Unparse(indent+4);
	ind(indent);
	cout <<  "}" << endl;
}

void CallStmtNode::Unparse(int indent)
{
	ind(indent);
	myId->Unparse();
	cout << "(" ;
	if(myExpList)
	{
		myExpList->Unparse();
	}
	cout << ");" << endl;
}

void ReturnStmtNode::Unparse(int indent)
{
	ind(indent);
	cout << "return;" << endl;
}

void IntLitNode::Unparse(int indent)
{
	cout << myIntVal;
}

void FloatLitNode::Unparse(int indent)
{
	cout << myFloatVal;
}

void StrLitNode::Unparse(int indent)
{
	cout << *myStrVal;
}

void TrueNode::Unparse(int indent)
{
	cout << "true";
}

void FalseNode::Unparse(int indent)
{
	cout << "false";
}

void UnaryMinusExpNode::Unparse(int indent)
{
	cout << "-";
	myExp->Unparse();
}

void NotExpNode::Unparse(int indent)
{
	cout << "!";
	myExp->Unparse();
}

void PlusExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " + " ;
	myExp2->Unparse();
	cout << ")";
}

void MinusExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " - " ;
	myExp2->Unparse();
	cout << ")";
}

void TimesExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " * " ;
	myExp2->Unparse();
	cout << ")";
}

void DivideExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " / " ;
	myExp2->Unparse();
	cout << ")";
}

void AndExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " && " ;
	myExp2->Unparse();
	cout << ")";
}

void OrExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " || " ;
	myExp2->Unparse();
	cout << ")";
}

void EqualsExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " == " ;
	myExp2->Unparse();
	cout << ")";
}

void NotEqualsExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " != " ;
	myExp2->Unparse();
	cout << ")";
}

void LessExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " < " ;
	myExp2->Unparse();
	cout << ")";
}

void GreaterExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " > " ;
	myExp2->Unparse();
	cout << ")";
}

void LessEqExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " <= " ;
	myExp2->Unparse();
	cout << ")";
}

void GreaterEqExpNode::Unparse(int indent)
{
	cout << "(";
	myExp1->Unparse();
	cout << " >= " ;
	myExp2->Unparse();
	cout << ")";
}
